###################################################################################################################
###################################################################################################################
#################          Log file and full codes for analyses contained in the paper              ###############
#################                Oxilia, Bortolini, Marciani, Menghi Sartorio  et. al.              ###############
#################                   'Direct evidence that late Neanderthal occupation               ###############
#################                   precedes a technological shift in southwestern Italy'           ###############
#################                                                                                   ###############
#################                   submitted to the Journal of Human Evolution                     ###############
#################                                                                                   ###############
#################                                  July 2021                                        ###############
#################                                                                                   ###############
#################                                   authors:                                        ###############
#################                          Jessica C. Menghi Sartorio,                              ###############
#################                              Eugenio Bortolini,                                   ###############
#################                                Gregorio Oxilia                                    ###############
#################                                                                                   ###############
#################                                    v.1.0                                          ###############
#################                                                                                   ###############
#################                  jessicacristina.menghisartorio@students.uniroma2.eu              ###############
#################                            eugenio.bortolini2@unibo.it                            ###############
#################                             gregorio.oxilia3@unibo.it                             ###############
#################                                                                                   ###############
###################################################################################################################
###################################################################################################################



####################### R_version_4.1.0 ####################################################

##required libraries
library(rgl) #v.0.106.8
library(shapes) #v.1.2.6
library(tripack) #v1.3.9.1
library(MASS) #v.7.3.54
library(lmtest) #v.0.9.38
library(ape) #v.5.5
library(ade4) #v.1.7.16
library(pls) #v.2.7.3
library(Morpho) #v.2.8
library(geomorph) #v.4.0.0
library(geometry) #v.0.4.5
library(car) #v.3.0.10
library(grDevices) #v.4.1.0
library(factoextra) #v.1.0.7
library(vegan) #v.2.5.7
library(randomForest) #v.4.6.14
library(mda) #v.0.5.2
library(RColorBrewer) #v.1.1.2
library(caret)#v.6.0.88
library(earth)#v.5.3.0
library(RVAideMemoire)#v.0.9.79


# set working directory (replace missing input with chosen path)
setwd('./xxxxxxxxxx/Mondragone_codes/DataTemplates_Code/')


####CROWN DATA
###load data
teeth.outline.crown <- as.matrix(read.table("Lower_dm2_crown_outline.txt",header=FALSE))
teeth.labels.crown <- as.matrix(read.table("Lower_dm2_label.Crown.txt",header=FALSE))
teeth.specie.crown <- as.matrix(read.table("Lower_dm2_specie.Crown.txt",header=FALSE))
SP.crown <-as.factor(as.matrix (teeth.specie.crown))

# set parameters (ka=n.of points/rows for each individual sample; ma=n.of columns for coordinates ; n=number of individuals/sample size)
ka <- 16;ma <- 2;n <- 58

#create dataset for analysis
teeth.dm2.crown.array <- array(0,c(ka,ma,n))
for(i in 0:(n-1)){teeth.dm2.crown.array[,,i+1]<-as.matrix(teeth.outline.crown[(ka*i+1):(ka*i+ka),])}

#scaling and centering
array.10.crown <- array(10,c(ka,ma,1))
teeth.dm2.crown.array.tran <- array(0,dim=c(ka,ma,n))
for (i in 1:n){teeth.dm2.crown.array.tran[,,i] <- teeth.dm2.crown.array[,,i] - array.10.crown[,,1]}
CS.crown <- rep(0,n)
for (i in 1:n){CS.crown[i] <- sqrt(sum((teeth.dm2.crown.array[,,i] - rep(1,ka)%*%t(apply(teeth.dm2.crown.array[,,i],2,mean)))^2))}
teeth.dm2.crown.array.tran.resc <- array(0,dim=c(ka,ma,n))
for (i in 1:n){teeth.dm2.crown.array.tran.resc[,,i] <- teeth.dm2.crown.array.tran[,,i]/CS.crown[i]}
crown.MEANshape <- matrix(0,ka,ma)
for(i in 1:ka){for(j in 1:ma){crown.MEANshape[i,j] <- mean(teeth.dm2.crown.array.tran.resc[i,j,])}}
teeth.dm2.crown.array.centered <- array(0,c(ka,ma,n))
for(i in 1:n) teeth.dm2.crown.array.centered[,,i] <- teeth.dm2.crown.array.tran.resc[,,i] - crown.MEANshape

#calculate procrustes distance
distanceS<-function(MATRIX1,MATRIX2){sqrt(sum(((MATRIX2 - MATRIX1)^2)))}
teeth.dm2.crown.centered.mat <- matrix(0,n,ma*ka)
for (i in 1:n){teeth.dm2.crown.centered.mat[i,] <- c(teeth.dm2.crown.array.centered[,,i])}
teeth.dm2.crown.array.centered.mh <- teeth.dm2.crown.array.centered [,,SP.crown=="mh"]
teeth.dm2.crown.array.centered.n <- teeth.dm2.crown.array.centered [,,SP.crown=="n"]
Procdistance.dm2.mh.crown<-matrix(0,length(SP.crown[SP.crown=="mh"]),1)
for (i in 1:length(SP.crown[SP.crown=="mh"])) {Procdistance.dm2.mh.crown[i,1]<-distanceS(as.matrix(teeth.dm2.crown.array.centered.mh [,,i]), as.matrix(crown.MEANshape[,]))}
Procdistance.dm2.n.crown<-matrix(0,length(SP.crown[SP.crown=="n"]),1)
for (i in 1:length(SP.crown[SP.crown=="n"])) {Procdistance.dm2.n.crown[i,1]<-distanceS(as.matrix(teeth.dm2.crown.array.centered.n [,,i]), as.matrix(crown.MEANshape[,]))}


#PCA
indet.center.mat<-teeth.dm2.crown.centered.mat[56:58,]
teeth.dm2.crown.centered.mat<-teeth.dm2.crown.centered.mat[-c(56:58),]
SP.crown.no.ind <-as.factor(as.matrix (teeth.specie.crown[-c(56:58)]))
res.pca <- prcomp(teeth.dm2.crown.centered.mat, scale = F)
groups <- SP.crown.no.ind
ind.sup <-indet.center.mat
ind.sup.coord <- predict(res.pca, newdata = ind.sup)
rownames(ind.sup.coord) <- teeth.labels.crown [56:58]


#plotting results of PCA
p <- fviz_pca_ind(res.pca,
             geom="point",
             pointsize = 2,
             col.ind = groups,
             palette = c("blue","red","forestgreen"),
             legend.title = "Groups",
             repel = TRUE,
             addEllipses = TRUE, ellipse.type = "convex"
             )
#PCA plot with supp. indivduals (RSS1 Original, RSS1 Restored, RSS2)
fviz_add(p, ind.sup.coord, color ="goldenrod1", addlabel = TRUE, labelsize = 4, pointsize = 2)


####Statistical analysis
#PERMANOVA with Bonferroni correction
PCA.outline.dm2_scores.crown <- res.pca$x[,1:8]
PCA.outline.dm2_scores_euclid.crown <- dist(PCA.outline.dm2_scores.crown[,-ncol(PCA.outline.dm2_scores.crown)])
set.seed(123)
pairwise.perm.manova(PCA.outline.dm2_scores_euclid.crown,groups,p.method = "bonferroni")

#Shapiro-Wilk (normality) & Fligner-Killeen (homogeneity of variances) test
factor.specie.reduced.crown <- as.character(SP.crown.no.ind)
factor.specie.reduced.crown[factor.specie.reduced.crown!="n"] <- "mh"
factor.specie.reduced.crown <- as.factor(factor.specie.reduced.crown)
for(i in 1: ncol(PCA.outline.dm2_scores.crown)){
  print(tapply(PCA.outline.dm2_scores.crown[,i],factor.specie.reduced.crown,shapiro.test))
  print(i)}
hom.test<-cbind(PCA.outline.dm2_scores.crown,factor.specie.reduced.crown)
cols <- 1:8;factor_cols <- 9
fligner <- sapply(cols, function(x) {fligner.test(hom.test[,x] ~ interaction(hom.test[,factor_cols[1]]))})
p.values <- apply(fligner, 2, function(x) {x$p.value})

####Classification
#setting up data and train control method
PCA.outline.dm2_scores.crown2<-as.data.frame(PCA.outline.dm2_scores.crown)
PCA.outline.dm2_scores.crown2<-cbind(factor.specie.reduced.crown,PCA.outline.dm2_scores.crown2)
PCA.outline.dm2_scores.crown2$factor.specie.reduced.crown<-as.factor(PCA.outline.dm2_scores.crown2$factor.specie.reduced.crown)
PCs_ind.crown<-ind.sup.coord[,1:8]

set.seed(123)
fitControl <- trainControl(method = "repeatedcv",classProbs = TRUE,
			   summaryFunction = twoClassSummary,
                           number = 10,
                           repeats = 10)

#FDA model training
set.seed(825)
fda_fit <- train(factor.specie.reduced.crown ~ ., data = PCA.outline.dm2_scores.crown2,
                  method = "fda", metric = "ROC",
                  tuneLength = 20,
                  trControl = fitControl)
confusionMatrix(fda_fit)

#MARS model training
set.seed(825)
mars_fit <- train(factor.specie.reduced.crown ~ ., data = PCA.outline.dm2_scores.crown2,
                  method = "earth", metric = "ROC",
                  tuneLength = 20,
	          trControl = fitControl)
confusionMatrix(mars_fit)

#Random Forest model training
set.seed(825)
rf_fit <- train(factor.specie.reduced.crown ~ ., data = PCA.outline.dm2_scores.crown2,
                  method = "rf", metric = "ROC",
	          trControl = fitControl)
confusionMatrix(rf_fit)

#output of class prediction (type = "raw") and posterior probabilities (type = "prob") for all the trained models
pc_fda<-predict(fda_fit, PCs_ind.crown, type = "raw")
pp_fda<-predict(fda_fit, PCs_ind.crown, type = "prob")
pc_m<-predict(mars_fit, PCs_ind.crown, type = "raw")
pp_m<-predict(mars_fit, PCs_ind.crown, type = "prob")
pc_rf<-predict(rf_fit, PCs_ind.crown, type = "raw")
pp_rf<-predict(rf_fit, PCs_ind.crown, type = "prob")

#barplot of posterior probabilities for all the trained models
par(mfrow=c(3,1),mar=c(2.3, 4, 2.3, 4),oma=c(3,3,3,3))
cols <- brewer.pal(9, "Blues")
barplot((t(pp_fda)),col=cols[4:5])
legend("top", c("mh", "n"), xpd = TRUE, horiz = TRUE, inset = c(1, 1), bty = "n", pch = c(16,16), col = cols[4:5], cex = 1)
cols <- brewer.pal(9, "Blues")
barplot((t(pp_m)),col=cols[4:5])
legend("top", c("mh", "n"), xpd = TRUE, horiz = TRUE, inset = c(1, 1), bty = "n", pch = c(16,16), col = cols[4:5], cex = 1)
cols <- brewer.pal(9, "Blues")
barplot((t(pp_rf)),col=cols[4:5])
legend("top", c("mh", "n"), xpd = TRUE, horiz = TRUE, inset = c(1, 1), bty = "n", pch = c(16,16), col = cols[4:5], cex = 1)





####CERVIX DATA
###load data
teeth.outline.cervix <- as.matrix(read.table("Lower_dm2_cervix_outline.txt",header=FALSE))
teeth.labels.cervix <- as.matrix(read.table("Lower_dm2_label.Cervix.txt",header=FALSE))
teeth.specie.cervix <- as.matrix(read.table("Lower_dm2_specie.Cervix.txt",header=FALSE))
SP.cervix <-as.factor(as.matrix (teeth.specie.cervix))

# set parameters (ka=n.of points/rows for each individual sample; ma=n.of columns for coordinates ; n=number of individuals/sample size)
ka <- 16;ma <- 2;n <- 55

#create dataset for analysis
teeth.dm2.cervix.array <- array(0,c(ka,ma,n))
for(i in 0:(n-1)){teeth.dm2.cervix.array[,,i+1]<-as.matrix(teeth.outline.cervix[(ka*i+1):(ka*i+ka),])}

#scaling and centering
array.10.cervix <- array(10,c(ka,ma,1))
teeth.dm2.cervix.array.tran <- array(0,dim=c(ka,ma,n))
for (i in 1:n){teeth.dm2.cervix.array.tran[,,i] <- teeth.dm2.cervix.array[,,i] - array.10.cervix[,,1]}
CS.cervix <- rep(0,n)
for (i in 1:n){CS.cervix[i] <- sqrt(sum((teeth.dm2.cervix.array[,,i] - rep(1,ka)%*%t(apply(teeth.dm2.cervix.array[,,i],2,mean)))^2))}
teeth.dm2.cervix.array.tran.resc <- array(0,dim=c(ka,ma,n))
for (i in 1:n){teeth.dm2.cervix.array.tran.resc[,,i] <- teeth.dm2.cervix.array.tran[,,i]/CS.cervix[i]}
cervix.MEANshape <- matrix(0,ka,ma)
for(i in 1:ka){for(j in 1:ma){cervix.MEANshape[i,j] <- mean(teeth.dm2.cervix.array.tran.resc[i,j,])}}
teeth.dm2.cervix.array.centered <- array(0,c(ka,ma,n))
for(i in 1:n) teeth.dm2.cervix.array.centered[,,i] <- teeth.dm2.cervix.array.tran.resc[,,i] - cervix.MEANshape

#calculate procrustes distance
distanceS<-function(MATRIX1,MATRIX2){sqrt(sum(((MATRIX2 - MATRIX1)^2)))}
teeth.dm2.cervix.centered.mat <- matrix(0,n,ma*ka)
for (i in 1:n){teeth.dm2.cervix.centered.mat[i,] <- c(teeth.dm2.cervix.array.centered[,,i])}
teeth.dm2.cervix.array.centered.mh <- teeth.dm2.cervix.array.centered [,,SP.cervix=="mh"]
teeth.dm2.cervix.array.centered.n <- teeth.dm2.cervix.array.centered [,,SP.cervix=="n"]
Procdistance.dm2.mh.cervix<-matrix(0,length(SP.cervix[SP.cervix=="mh"]),1)
for (i in 1:length(SP.cervix[SP.cervix=="mh"])) {Procdistance.dm2.mh.cervix[i,1]<-distanceS(as.matrix(teeth.dm2.cervix.array.centered.mh [,,i]), as.matrix(cervix.MEANshape[,]))}
Procdistance.dm2.n.cervix<-matrix(0,length(SP.cervix[SP.cervix=="n"]),1)
for (i in 1:length(SP.cervix[SP.cervix=="n"])) {Procdistance.dm2.n.cervix[i,1]<-distanceS(as.matrix(teeth.dm2.cervix.array.centered.n [,,i]), as.matrix(cervix.MEANshape[,]))}


#PCA
indet.center.mat.cervix<-teeth.dm2.cervix.centered.mat[54:55,]
teeth.dm2.cervix.centered.mat<-teeth.dm2.cervix.centered.mat[-c(54:55),]
SP.cervix.no.ind <-as.factor(as.matrix (teeth.specie.cervix[-c(54:55)]))
res.pca.cervix <- prcomp(teeth.dm2.cervix.centered.mat, scale = F)
groups.cervix <- SP.cervix.no.ind
ind.sup.cervix <-indet.center.mat.cervix
ind.sup.coord.cervix <- predict(res.pca.cervix, newdata = ind.sup.cervix)
rownames(ind.sup.coord.cervix) <- teeth.labels.cervix [54:55]

#plotting results of PCA
p.cervix <- fviz_pca_ind(res.pca.cervix,
             geom="point",
	           pointsize = 2,
             col.ind = groups.cervix,
             palette= c("blue","red","forestgreen"),
             legend.title = "Groups",
             repel = TRUE,
             addEllipses = TRUE, ellipse.type = "convex"
             )
#PCA plot with supp. indivduals (RSS1 Original, RSS2)
fviz_add(p.cervix, ind.sup.coord.cervix, color ="goldenrod1", addlabel = TRUE, labelsize = 4, pointsize = 2, repel=TRUE)


####Statistical analysis
#PERMANOVA with Bonferroni correction
PCA.outline.dm2_scores.cervix <- res.pca.cervix$x[,1:8]
PCA.outline.dm2_scores_euclid.cervix <- dist(PCA.outline.dm2_scores.cervix[,-ncol(PCA.outline.dm2_scores.cervix)])
set.seed(12)
pairwise.perm.manova(PCA.outline.dm2_scores_euclid.cervix,groups.cervix,p.method = "bonferroni")

#Shapiro-Wilk (normality) & Fligner-Killeen (homogeneity of variances) test
factor.specie.reduced.cervix <- as.character(SP.cervix.no.ind)
factor.specie.reduced.cervix[factor.specie.reduced.cervix!="n"] <- "mh"
factor.specie.reduced.cervix <- as.factor(factor.specie.reduced.cervix)
for(i in 1: ncol(PCA.outline.dm2_scores.cervix)){
  print(tapply(PCA.outline.dm2_scores.cervix[,i],factor.specie.reduced.cervix,shapiro.test))
  print(i)}
hom.test.cervix<-cbind(PCA.outline.dm2_scores.cervix,factor.specie.reduced.cervix)
cols.cervix <- 1:8;factor_cols.cervix <- 9
fligner.cervix <- sapply(cols.cervix, function(x) {fligner.test(hom.test.cervix[,x] ~ interaction(hom.test.cervix[,factor_cols.cervix[1]]))})
p.values.cervix <- apply(fligner.cervix, 2, function(x) {x$p.value})

####Classification
#setting up data and train control method
PCA.outline.dm2_scores.cervix2<-as.data.frame(PCA.outline.dm2_scores.cervix)
PCA.outline.dm2_scores.cervix2<-cbind(factor.specie.reduced.cervix,PCA.outline.dm2_scores.cervix2)
PCA.outline.dm2_scores.cervix2$factor.specie.reduced.cervix<-as.factor(PCA.outline.dm2_scores.cervix2$factor.specie.reduced.cervix)
PCs_ind.cervix<-ind.sup.coord.cervix[,1:8]

set.seed(123)
fitControl <- trainControl(method = "repeatedcv",classProbs = TRUE,
			   summaryFunction = twoClassSummary,
                           number = 10,
                           repeats = 10)

#FDA model training
set.seed(825)
fda_fit_cervix <- train(factor.specie.reduced.cervix ~ ., data = PCA.outline.dm2_scores.cervix2,
                  method = "fda", metric = "ROC",
                  tuneLength = 20,
                  trControl = fitControl)
confusionMatrix(fda_fit_cervix)

#MARS model training
set.seed(825)
mars_fit_cervix <- train(factor.specie.reduced.cervix ~ ., data = PCA.outline.dm2_scores.cervix2,
                  method = "earth", metric = "ROC",
                  tuneLength = 20,
	          trControl = fitControl)
confusionMatrix(mars_fit_cervix)

#Random Forest model training
set.seed(825)
rf_fit_cervix <- train(factor.specie.reduced.cervix ~ ., data = PCA.outline.dm2_scores.cervix2,
                  method = "rf", metric = "ROC",
	          trControl = fitControl)
confusionMatrix(rf_fit_cervix)

#output of class prediction (type = "raw") and posterior probabilities (type = "prob") for all the trained models
pc_fda_cervix<-predict(fda_fit_cervix, PCs_ind.cervix, type = "raw")
pp_fda_cervix<-predict(fda_fit_cervix, PCs_ind.cervix, type = "prob")
pc_m_cervix<-predict(mars_fit_cervix, PCs_ind.cervix, type = "raw")
pp_m_cervix<-predict(mars_fit_cervix, PCs_ind.cervix, type = "prob")
pc_rf_cervix<-predict(rf_fit_cervix, PCs_ind.cervix, type = "raw")
pp_rf_cervix<-predict(rf_fit_cervix, PCs_ind.cervix, type = "prob")

#barplot of posterior probabilities for all the trained models
par(mfrow=c(3,1),mar=c(2.3, 4, 2.3, 4),oma=c(3,3,3,3))
cols <- brewer.pal(9, "Blues")
barplot((t(pp_fda_cervix)),col=cols[4:5])
legend("top", c("mh", "n"), xpd = TRUE, horiz = TRUE, inset = c(1, 1), bty = "n", pch = c(16,16), col = cols[4:5], cex = 1)
cols <- brewer.pal(9, "Blues")
barplot((t(pp_m_cervix)),col=cols[4:5])
legend("top", c("mh", "n"), xpd = TRUE, horiz = TRUE, inset = c(1, 1), bty = "n", pch = c(16,16), col = cols[4:5], cex = 1)
cols <- brewer.pal(9, "Blues")
barplot((t(pp_rf_cervix)),col=cols[4:5])
legend("top", c("mh", "n"), xpd = TRUE, horiz = TRUE, inset = c(1, 1), bty = "n", pch = c(16,16), col = cols[4:5], cex = 1)






















